@@ -1,4 +1,4 @@
-/*  $Id: init.pl,v 1.11 1994/01/31 09:53:51 jan Exp $
+/*  init.pl,v 1.14 1994/11/24 14:06:39 jan Exp
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     jan@swi.psy.uva.nl
@@ -45,35 +45,35 @@ op(Priority, Type, Name) :-
 
 dynamic((Pred/Arity, More)) :- !,
 	functor(Term, Pred, Arity),
-	$predicate_attribute(Term, (dynamic), 1),
+	$set_predicate_attribute(Term, (dynamic), 1),
 	dynamic(More).
 dynamic(Pred/Arity) :-
 	functor(Term, Pred, Arity),
-	$predicate_attribute(Term, (dynamic), 1).
+	$set_predicate_attribute(Term, (dynamic), 1).
 
 multifile((Pred/Arity, More)) :- !,
 	functor(Term, Pred, Arity),
-	$predicate_attribute(Term, (multifile), 1),
+	$set_predicate_attribute(Term, (multifile), 1),
 	multifile(More).
 multifile(Pred/Arity) :-
 	functor(Term, Pred, Arity),
-	$predicate_attribute(Term, (multifile), 1).
+	$set_predicate_attribute(Term, (multifile), 1).
 
 module_transparent((Pred/Arity, More)) :- !,
 	functor(Term, Pred, Arity),
-	$predicate_attribute(Term, transparent, 1),
+	$set_predicate_attribute(Term, transparent, 1),
 	module_transparent(More).
 module_transparent(Pred/Arity) :-
 	functor(Term, Pred, Arity),
-	$predicate_attribute(Term, transparent, 1).
+	$set_predicate_attribute(Term, transparent, 1).
 
 discontiguous((Pred/Arity, More)) :- !,
 	functor(Term, Pred, Arity),
-	$predicate_attribute(Term, (discontiguous), 1),
+	$set_predicate_attribute(Term, (discontiguous), 1),
 	discontiguous(More).
 discontiguous(Pred/Arity) :-
 	functor(Term, Pred, Arity),
-	$predicate_attribute(Term, (discontiguous), 1).
+	$set_predicate_attribute(Term, (discontiguous), 1).
 
 :- module_transparent
 	(dynamic)/1,
@@ -93,7 +93,7 @@ discontiguous(Pred/Arity) :-
 
 $hide(Name, Arity) :-
 	functor(Head, Name, Arity),
-	$predicate_attribute(Head, trace, 0).
+	$set_predicate_attribute(Head, trace, 0).
 
 %	$show_childs(+Name, +Arity)
 %	Normally system predicates hide their childs frames if these are
@@ -101,7 +101,7 @@ $hide(Name, Arity) :-
 
 $show_childs(Name, Arity) :-  
 	functor(Head, Name, Arity),
-        $predicate_attribute(Head, hide_childs, 0).
+        $set_predicate_attribute(Head, hide_childs, 0).
 
 		/********************************
 		*       CALLING, CONTROL        *
@@ -705,6 +705,14 @@ $assert_module_file(File, Module) :-
 	asserta($module_file(File, Module)).
 
 
+$reserved_module(system).
+$reserved_module(user).
+
+$load_module(Reserved, _, _, _) :-
+	$reserved_module(Reserved), !,
+	$warning('Cannot load into module "~w": reserved module name',
+		 [Reserved]),
+	fail.
 $load_module(Module, Public, Import, File) :-
 	$set_source_module(OldModule, OldModule),
 	(   compiling
@@ -1008,6 +1016,14 @@ append([H|T], L, [H|R]) :-
 	append(T, L, R).
 
 
+		 /*******************************
+		 *	       MISC		*
+		 *******************************/
+
+halt :-
+	halt(0).
+
+
 		/********************************
 		*            EXPORTS            *
 		*********************************/
