@@ -1,4 +1,4 @@
-/*  $Id: pl-alloc.c,v 1.15 1994/10/20 08:52:36 jan Exp $
+/*  pl-alloc.c,v 1.16 1994/11/24 14:07:12 jan Exp
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -315,13 +315,6 @@ unpack_real(Word p)
 }
 
 
-void
-setReal(word w, real f)
-{ Word p = (Word)unMask(w);
-  pack_real((double)f, p);
-}
-
-
 word
 globalReal(real f)
 { Word p = gTop;
@@ -334,6 +327,7 @@ globalReal(real f)
   return (word)p | INDIRECT_MASK;
 }
 
+
 word
 heapReal(real f)
 { Word p = (Word) allocHeap(sizeof(word) * 2);
