@@ -1,4 +1,4 @@
-/*  $Id: pl-atom.c,v 1.9 1994/03/02 14:32:56 jan Exp $
+/*  pl-atom.c,v 1.12 1994/11/24 14:07:14 jan Exp
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -140,7 +140,7 @@ allAlpha(register char *s)
 }
 
 
-char *
+static char *
 extendAtom(char *prefix, bool *unique)
 { Atom a = atomTable[0];
   bool first = TRUE;
@@ -202,7 +202,7 @@ compareMatch(const void *m1, const void *m2)
 }
 
 
-bool
+static bool
 extend_alternatives(char *prefix, struct match *altv, int *altn)
 { Atom a = atomTable[0];
   char *as;
@@ -257,17 +257,7 @@ pl_atom_completions(Word prefix, Word alts)
 } 
 
 
-#if O_READLINE
-
-static char *
-xmalloc(int size)
-{ char *result = malloc(size);
-
-  if ( !result )
-    fatalError("Not enough core");
-
-  return result;
-}
+#ifdef HAVE_LIBREADLINE
 
 #define savestring(x) strcpy(xmalloc(1 + strlen(x)), (x))
 
@@ -303,4 +293,4 @@ atom_generator(char *prefix, int state)
   return NULL;
 }
 
-#endif /*O_READLINE*/
+#endif /*HAVE_LIBREADLINE*/
