@@ -1,4 +1,4 @@
-/*  $Id: pl-comp.c,v 1.11 1994/03/02 14:32:59 jan Exp $
+/*  pl-comp.c,v 1.16 1994/12/02 16:40:22 jan Exp
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -147,7 +147,7 @@ NOTE:	If the assert() fails, look at pl-wam.c: VMI(C_NOT, ... for
 	more information.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#if O_VMCODE_IS_ADDRESS
+#if VMCODE_IS_ADDRESS
 void
 initWamTable(void)
 { int n;
@@ -175,14 +175,14 @@ initWamTable(void)
   checkCodeTable();
 }
 
-#else /* O_VMCODE_IS_ADDRESS */
+#else /* VMCODE_IS_ADDRESS */
 
 void
 initWamTable()
 { checkCodeTable();
 }
 
-#endif /* O_VMCODE_IS_ADDRESS */
+#endif /* VMCODE_IS_ADDRESS */
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 This module forms together  with  the  module  'pl-wam.c'  the  complete
@@ -405,7 +405,7 @@ calculation at runtime.
 #define PC(ci)		entriesBuffer(&(ci)->codes, code)
 #define OpCode(ci, pc)	(baseBuffer(&(ci)->codes, code)[pc])
 
-struct vartable
+static struct vartable
 { int	entry[MAXVARIABLES/BITSPERINT];
 } empty_var_table;
 
@@ -505,6 +505,7 @@ setVars(register Word t, register struct vartable *vt)
   }
 }
 
+
 static Clause
 compile(Word term, Module module)
 { compileInfo ci;			/* data base for the compiler */
@@ -523,13 +524,15 @@ clause should belong to.
     return (Clause) NULL;
   }
   if (isAtom(*head) )
-    proc = lookupProcedure(lookupFunctorDef((Atom)*head, 0), module);
+    proc = lookupProcedureToDefine(lookupFunctorDef((Atom)*head, 0), module);
   else if (isTerm(*head) )
-    proc = lookupProcedure(functorTerm(*head), module);
+    proc = lookupProcedureToDefine(functorTerm(*head), module);
   else
   { warning("compiler: illegal clause head");
     return (Clause) NULL;
   }
+  if ( !proc )
+    return NULL;
 
   if ( (ci.arity = proc->functor->arity) > MAXARITY )
     return (Clause) warning("Compiler: arity too high (%d)\n", ci.arity);
@@ -547,6 +550,7 @@ Allocate the clause and fill initialise the field we already know.
   clause->XR_size = clause->code_size = 0;
   clause->subclauses = 0;
   clause->procedure = proc;
+  clause->source_no = clause->line_no = 0;
 
   DEBUG(9, printf("clause struct initialised\n"));
 
@@ -560,7 +564,7 @@ Allocate the clause and fill initialise the field we already know.
   }
 
   TRY( analyse_variables(head, body, ci.arity, &nvars) );
-  clause->variables = clause->slots = nvars + ci.arity;
+  clause->variables = nvars + ci.arity;
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Initialise the `compileInfo' structure.
@@ -1248,9 +1252,8 @@ assert_term(Word term, char where, Atom file)
 { Clause clause;
   Procedure proc;
   Definition def;
-  Module source_module = (file != (Atom) NULL ? modules.source : (Module) NULL);
+  Module source_module = (file ? modules.source : (Module) NULL);
   Module module = source_module;
-  static Procedure current = (Procedure) NULL;
 
   term = stripModule(term, &module);
 
@@ -1258,6 +1261,7 @@ assert_term(Word term, char where, Atom file)
   if ((clause = compile(term, module)) == (Clause) NULL)
     return (Clause) NULL;
   DEBUG(9, printf("ok\n"));
+  clause->line_no = source_line_no;
   proc = clause->procedure;
   def = proc->definition;
 
@@ -1266,22 +1270,14 @@ If file is defined, we are called from record_clause/2.  This code takes
 care of reconsult, redefinition, etc.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-  if (file != (Atom) NULL)
+  if ( file )
   { SourceFile sf;
 
     sf = lookupSourceFile(file);
+    clause->line_no   = source_line_no;
+    clause->source_no = sf->index;
 
-    if ( true(def, MULTIFILE) )
-    { if ( sf->count != 1 )
-      { warning("Multifile predicate %s not updated", procedureName(proc) );
-	freeClause(clause);
-	return (Clause) NULL;
-      }
-      return assertProcedure(proc, clause, where) == FALSE ? (Clause) NULL
-							   : clause;
-    }
-
-    if (def->module != module)
+    if ( def->module != module )
     { if ( true(def->module, SYSTEM) )
         warning("Attempt to redefine a system predicate: %s", 
 				procedureName(proc));
@@ -1291,80 +1287,62 @@ care of reconsult, redefinition, etc.
 				proc->functor->arity, 
 				stringAtom(proc->definition->module->name) );
       freeClause(clause);
-      return (Clause) NULL;
+      return NULL;
     }
 
-    if (def->source == sf && def->source_count == sf->count)
-    { if (proc != current)
-      { if ( (debugstatus.styleCheck & DISCONTIGUOUS_STYLE) &&
-	     false(def, DISCONTIGUOUS) )
-	  warning("Clauses of %s are not together in the source file", 
-				procedureName(proc) );
-	current = proc;
+    if ( proc == sf->current_procedure )
+      return assertProcedure(proc, clause, where) ? clause : NULL;
+
+    if ( def->definition.clauses )	/* i.e. is defined */
+    { Clause first;
+
+      if ( true(def, LOCKED) && !SYSTEM_MODE && false(def, DYNAMIC|MULTIFILE) )
+      { warning("Attempt to redefine a system predicate: %s",
+		procedureName(proc));
+	freeClause(clause);
+	return NULL;
       }
 
-      return assertProcedure(proc, clause, where) == FALSE ? (Clause) NULL
-							   : clause;
-    } else
-      def->line_no = source_line_no;
+      if ( true(def, FOREIGN) )
+      { abolishProcedure(proc, module);
+	warning("Redefined: foreign predicate %s", procedureName(proc));
+      }
 
-    current = proc;
+      if ( false(def, MULTIFILE) )
+      { Clause first = def->definition.clauses;
+
+	if ( first && first->source_no == sf->index )
+	{ if ( (debugstatus.styleCheck & DISCONTIGUOUS_STYLE) &&
+	       false(def, DISCONTIGUOUS) )
+	    warning("Clauses of %s are not together in the source file", 
+		    procedureName(proc));
+	} else
+	{ abolishProcedure(proc, module);
+	  warning("Redefined: %s", procedureName(proc));
+	}
+      }
+
+      addProcedureSourceFile(sf, proc);
+      sf->current_procedure = proc;
+      
+      return assertProcedure(proc, clause, where) ? clause : NULL;
+    }
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 This `if' locks predicates as system predicates  if  we  are  in  system
 mode, the predicate is still undefined and is not dynamic or multifile.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-    if ( SYSTEM_MODE &&
-	 false(def, SYSTEM) &&
-	 false(def, DYNAMIC) &&
-	 false(def, MULTIFILE) &&
-	 def->definition.clauses == (Clause) NULL)
-    { set(def, SYSTEM);
-      set(def, HIDE_CHILDS);
+    if ( SYSTEM_MODE && false(def, SYSTEM) )
+      set(def, SYSTEM|HIDE_CHILDS|LOCKED);
 
-      def->source = sf;
-      def->source_count = sf->count;
-
-      return assertProcedure(proc, clause, where) == FALSE ? (Clause) NULL
-							   : clause;
-    }
-
-    if ( true(def, SYSTEM) && !SYSTEM_MODE )
-    { warning("Attempt to redefine a system predicate: %s", 
-				procedureName(proc) );
-      freeClause(clause);
-      return (Clause) NULL;
-    }
-
-    if (def->source != sf)
-    { if (def->definition.clauses != (Clause) NULL)
-      {	abolishProcedure(proc, module);
-	warning("Redefined: %s", procedureName(proc) );
-      }
-      def->source = sf;
-      def->source_count = sf->count;
-    } else
-    { if (def->source_count < sf->count)	/* reconsult */
-      { removeClausesProcedure(proc);
-	def->line_no = source_line_no;
-        def->source = sf;
-      	def->source_count = sf->count;
-      }
-    }
-    return assertProcedure(proc, clause, where) == FALSE ? (Clause) NULL
-							 : clause;
+    addProcedureSourceFile(sf, proc);
+    sf->current_procedure = proc;
+    return assertProcedure(proc, clause, where) ? clause : NULL;
   }
 
   /* assert[az]/1 */
 
-  if ( true(def, SYSTEM) && false(def, DYNAMIC) )
-  { warning("Attempt to redefine a system predicate: %s", 
-				procedureName(proc) );
-    freeClause(clause);
-    return (Clause) NULL;
-  }
-
   if ( def->module != module && false(def, DYNAMIC) )
   { warning("Attempt to redefine an imported predicate %s", 
 			      procedureName(proc) );
@@ -1387,6 +1365,7 @@ pl_asserta(Word term)
 { return assert_term(term, 'a', (Atom)NULL) == (Clause)NULL ? FALSE : TRUE;
 }
 
+
 word
 pl_assertz2(Word term, Word ref)
 { Clause clause = assert_term(term, 'z', (Atom)NULL);
@@ -1397,6 +1376,7 @@ pl_assertz2(Word term, Word ref)
   return unifyAtomic(ref, pointerToNum(clause));
 }
 
+
 word
 pl_asserta2(Word term, Word ref)
 { Clause clause = assert_term(term, 'a', (Atom)NULL);
@@ -1407,6 +1387,7 @@ pl_asserta2(Word term, Word ref)
   return unifyAtomic(ref, pointerToNum(clause));
 }
 
+
 word
 pl_record_clause(Word term, Word file)
 { if (!isAtom(*file) )
@@ -1416,6 +1397,29 @@ pl_record_clause(Word term, Word file)
 }  
 
 
+word
+pl_redefine_system_predicate(Word head)
+{ Module m = NULL;
+  FunctorDef def;
+  Procedure proc;
+
+  if ( (head = stripModule(head, &m)) == NULL )
+    fail;
+
+  if ( isAtom(*head) )
+    def = lookupFunctorDef((Atom)*head, 0);
+  else if ( isTerm(*head) )
+    def = functorTerm(*head);
+  else
+    return warning("redefine_system_predicate/1: instantiation fault");
+
+  proc = lookupProcedure(def, m);
+  abolishProcedure(proc, m);
+
+  succeed;
+}
+
+
 		/********************************
 		*          DECOMPILER           *
 		*********************************/
