@@ -1,4 +1,4 @@
-/*  $Id: pl-dump.c,v 1.13 1994/06/20 10:44:36 jan Exp $
+/*  pl-dump.c,v 1.14 1994/11/04 14:19:00 jan Exp
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -202,7 +202,7 @@ pl_save(Word file, Word restore)
   { case SAVE_SAVE:
       return unifyAtomic(restore, consNum(0));
     case SAVE_RESTORE:
-#if unix || EMX
+#ifdef HAVE_SIGNAL
       initSignals();
 #endif
       return unifyAtomic(restore, consNum(1));
