@@ -1,4 +1,4 @@
-/*  $Id: pl-ext.c,v 1.19 1994/10/19 15:39:20 jan Exp $
+/*  pl-ext.c,v 1.23 1994/11/30 10:29:57 jan Exp
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -41,6 +41,7 @@ static struct foreign {
   ADD("tmp_file",		2, pl_tmp_file,			TRACE_ME),
   ADD("delete_file",		1, pl_delete_file,		TRACE_ME),
   ADD("access_file",		2, pl_access_file,		TRACE_ME),
+  ADD("read_link",		3, pl_read_link,		TRACE_ME),
   ADD("exists_file",		1, pl_exists_file,		TRACE_ME),
   ADD("exists_directory",	1, pl_exists_directory,		TRACE_ME),
   ADD("rename_file",		2, pl_rename_file,		TRACE_ME),
@@ -59,10 +60,11 @@ static struct foreign {
   ADD("absolute_file_name",	2, pl_absolute_file_name,	TRACE_ME),
   ADD("$file_base_name",	2, pl_file_base_name,		TRACE_ME),
   ADD("$file_dir_name",		2, pl_file_dir_name,		TRACE_ME),
+  ADD("prolog_to_os_filename",	2, pl_prolog_to_os_filename,	TRACE_ME),
   ADD("fileerrors",		2, pl_fileerrors,		TRACE_ME),
   ADD("chdir",			1, pl_chdir,			TRACE_ME),
 
-  ADD("halt",			0, pl_halt,			TRACE_ME),
+  ADD("halt",			1, pl_halt,			TRACE_ME),
   ADD("$shell",			2, pl_shell,			TRACE_ME),
   ADD("getenv",			2, pl_getenv,			TRACE_ME),
   ADD("setenv",			2, pl_setenv,			TRACE_ME),
@@ -137,9 +139,13 @@ static struct foreign {
   ADD("asserta",		2, pl_asserta2,		TRANSPARENT|TRACE_ME),
   ADD("assertz",		2, pl_assertz2,		TRANSPARENT|TRACE_ME),
   ADD("$record_clause",		2, pl_record_clause,		TRACE_ME),
+  ADD("redefine_system_predicate", 1, pl_redefine_system_predicate,
+							TRANSPARENT|TRACE_ME),
 
   ADD("$c_current_predicate",	2, pl_current_predicate,  NONDETERMINISTIC|TRANSPARENT|TRACE_ME),
-  ADD("$predicate_attribute",	3, pl_predicate_attribute,TRANSPARENT|TRACE_ME),
+  ADD("$set_predicate_attribute", 3, pl_set_predicate_attribute,TRANSPARENT|TRACE_ME),
+  ADD("$get_predicate_attribute", 3, pl_get_predicate_attribute,TRANSPARENT|TRACE_ME),
+  ADD("$get_clause_attribute",  3, pl_get_clause_attribute,	TRACE_ME),
   ADD("$source_file",		2, pl_source_file,	  TRANSPARENT|TRACE_ME),
   ADD("$time_source_file",	2, pl_time_source_file,	  NONDETERMINISTIC|TRACE_ME),
   ADD("$start_consult",		1, pl_start_consult,		TRACE_ME),
@@ -243,13 +249,13 @@ static struct foreign {
 #else
   ADD("$load_foreign",		5, pl_load_foreign,	TRANSPARENT|TRACE_ME),
 #endif
-#if O_DLOPEN
+#if HAVE_DLOPEN
   ADD("open_shared_object",	2, pl_open_shared_object,	TRACE_ME),
   ADD("close_shared_object",	1, pl_close_shared_object,	TRACE_ME),
   ADD("call_shared_object_function",
 				2, pl_call_shared_object_function,
 							TRANSPARENT|TRACE_ME),
-#endif /*O_DLOPEN*/
+#endif /*HAVE_DLOPEN*/
 
 #if O_DDE
   ADD("open_dde_conversation",	3, pl_open_dde_conversation,	TRACE_ME),
@@ -268,7 +274,6 @@ static struct foreign {
   ADD("save",			2, pl_save,			TRACE_ME),
   ADD("restore",		1, pl_restore,			TRACE_ME),
   ADD("$save_program",		2, pl_save_program,		TRACE_ME),
-  ADD("$version",		1, pl_version,			TRACE_ME),
 
   ADD("is_list",		1, pl_is_list,			TRACE_ME),
   ADD("proper_list",		1, pl_proper_list,		TRACE_ME),
@@ -286,9 +291,8 @@ static struct foreign {
 #endif /* O_COMPILE_OR */
   ADD("$atom_hashstat",		2, pl_atom_hashstat,		TRACE_ME),
   ADD("$tty",			0, pl_tty,			TRACE_ME),
-  ADD("$arch",			2, pl_arch,			TRACE_ME),
   ADD("$grep",			3, pl_grep,	NONDETERMINISTIC|TRACE_ME),
-  ADD("$home",			1, pl_home,			TRACE_ME),
+  ADD("feature",		2, pl_feature,	NONDETERMINISTIC|TRACE_ME),
   ADD("limit_stack",		2, pl_limit_stack,		TRACE_ME),
   ADD("trim_stacks",		0, pl_trim_stacks,		TRACE_ME),
 #if O_SHIFT_STACKS
@@ -356,7 +360,7 @@ initBuildIns(void)
   for(f = &foreigns[0]; f->name; f++)
   { def = lookupProcedure(lookupFunctorDef(lookupAtom(f->name), f->arity), 
 					         MODULE_system)->definition;
-    set(def, FOREIGN|SYSTEM);
+    set(def, FOREIGN|SYSTEM|LOCKED);
     clear(def, TRACE_ME);
     set(def, f->flags);
     def->definition.function = f->function;
