@@ -1,4 +1,4 @@
-/*  $Id: pl-fmt.c,v 1.7 1994/03/02 14:33:05 jan Exp $
+/*  pl-fmt.c,v 1.8 1994/11/04 14:19:04 jan Exp
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -160,7 +160,7 @@ va_list args;
 { 
 }
 
-#if ANSI
+
 bool
 format(char *fm, ...)
 { va_list args;
@@ -173,24 +173,6 @@ format(char *fm, ...)
   return rval;
 }
 
-#else
-
-bool
-format(va_alist)
-va_dcl
-{ va_list args;
-  char *fm;
-  bool rval;
-
-  va_start(args);
-  fm = va_arg(args, char *);
-  rval = vformat(fm, args);
-  va_end(args);
-
-  return rval;
-}
-#endif /* ANSI */
-
 #endif /* O_C_FORMAT */
 		/********************************
 		*       ACTUAL FORMATTING	*
