@@ -1,4 +1,4 @@
-/*  $Id: pl-funct.c,v 1.4 1994/03/02 14:33:07 jan Exp $
+/*  pl-funct.c,v 1.5 1994/11/24 14:07:24 jan Exp
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -41,20 +41,6 @@ lookupFunctorDef(register Atom atom, register int arity)
 }
 
 
-int
-atomIsFunctor(Atom atom)
-{ int v = pointerHashValue(atom, FUNCTORHASHSIZE);
-  FunctorDef f;
-
-  for(f = functorDefTable[v]; f && !isRef((word)f); f = f->next)
-  { if ( atom == f->name )
-      return f->arity;
-  }
-
-  return -1;
-}
-
-
 FunctorDef
 isCurrentFunctor(Atom atom, int arity)
 { int v = pointerHashValue(atom, FUNCTORHASHSIZE);
@@ -69,7 +55,7 @@ isCurrentFunctor(Atom atom, int arity)
 }
 
 
-bool
+static bool
 atomIsProcedureModule(Atom atom, Module m)
 { int v = pointerHashValue(atom, FUNCTORHASHSIZE);
   FunctorDef f;
@@ -86,18 +72,6 @@ atomIsProcedureModule(Atom atom, Module m)
 }
 
 
-bool
-atomIsProcedure(Atom atom)
-{ Symbol s;
-
-  for_table(s, moduleTable)
-    if ( atomIsProcedureModule(atom, (Module)s->value) )         
-      succeed;
-
-  fail;
-}
-
-
 struct functorDef functors[] = {
 #include "pl-funct.ic"
 { (FunctorDef)NULL,	FUNCTOR_TYPE,	(Atom) NULL, 0 }
