@@ -1,4 +1,4 @@
-/*  $Id: pl-index.c,v 1.4 1994/03/02 14:33:10 jan Exp $
+/*  pl-index.c,v 1.6 1994/11/07 09:47:43 jan Exp
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -46,7 +46,11 @@ indexing only on the first argument as this is default.
 
 static unsigned long variable_mask[][4] =
   { { 0,        0,        0,        0 }, 
-    { VM(1, 0), 0,        0,        0 }, 
+#ifdef DONOT_AVOID_SHIFT_WARNING
+    { VM(1, 0), 0,        0,        0 },
+#else
+    { ~0L,      0,        0,        0 },
+#endif
     { VM(2, 0), VM(2, 1), 0,        0 }, 
     { VM(3, 0), VM(3, 1), VM(3, 2), 0 }, 
     { VM(4, 0), VM(4, 1), VM(4, 2), VM(4, 3) }
@@ -61,7 +65,13 @@ static int mask_shift[][4] =
   };
 
 static unsigned long mask_mask[] =
-  { 0, MASK(1), MASK(2), MASK(3), MASK(4)
+  { 0,
+#ifdef DONOT_AVOID_SHIFT_WARNING
+    MASK(1),
+#else
+    0L,
+#endif
+    MASK(2), MASK(3), MASK(4)
   };
 
 /*  Determine cardinality (= # 1's) of bit pattern.
