@@ -1,4 +1,4 @@
-/*  $Id: pl-itf.c,v 1.17 1994/10/19 15:15:01 jan Exp $
+/*  pl-itf.c,v 1.21 1994/11/30 10:30:03 jan Exp
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -239,13 +239,14 @@ registerForeign(char *name, int arity, Func f, va_list args)
   proc = lookupProcedure(lookupFunctorDef(lookupAtom(name), arity), m);
   def = proc->definition;
 
-  if ( true(def, SYSTEM) )
-    return warning("PL_register_foreign(): Attempt to redefine a system predicate: %s",
-		   procedureName(proc));
-  if ( def->source != (SourceFile) NULL && def->source != sf )
-    warning("PL_register_foreign(): redefined %s", procedureName(proc));
-  def->source = sf;
+  if ( true(def, LOCKED) )
+  { warning("PL_register_foreign(): Attempt to redefine a system predicate: %s",
+	    procedureName(proc));
+    fail;
+  }
 
+  if ( def->definition.function )
+    warning("PL_register_foreign(): redefined %s", procedureName(proc));
   if ( false(def, FOREIGN) && def->definition.clauses != (Clause) NULL )
     abolishProcedure(proc, m);
 
@@ -346,7 +347,7 @@ PL_new_module(register atomic name)
 		*            SIGNALS            *
 		*********************************/
 
-#if O_SIGNAL
+#if HAVE_SIGNAL
 void
 (*PL_signal(int sig, void (*func) (/* ??? */)))(/* ??? */)
 { void (*old)();
@@ -451,7 +452,8 @@ PL_action(int action, void *arg)
     case PL_ACTION_BREAK:
       return (bool) pl_break();
     case PL_ACTION_HALT:
-      return (bool) pl_halt();
+      Halt((int) arg);
+      fail;				/* should not happen */
     case PL_ACTION_ABORT:
       return (bool) pl_abort();
     case PL_ACTION_SYMBOLFILE:
