@@ -1,4 +1,4 @@
-/*  $Id: pl-pro.c,v 1.12 1994/03/02 14:33:21 jan Exp $
+/*  pl-pro.c,v 1.15 1994/12/02 16:40:25 jan Exp
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -86,8 +86,6 @@ callGoal(Module module, word goal, bool debug)
 
   lockp(&lSave);
   lockp(&envSave);
-/*lTop = (LocalFrame) addPointer(lTop, sizeof(struct localFrame) +
-				 MAXARITY * sizeof(word));*/
   lTop = (LocalFrame) addPointer(lTop, sizeof(word));
   verifyStack(local);
   varFrame(lTop, -1) = (word) environment_frame;
@@ -126,7 +124,7 @@ pl_abort()
   closeFiles();
   resetReferences();
   resetForeign();
-#if O_PROFILE
+#ifdef O_PROFILE
   pl_reset_profiler();
 #endif
 
@@ -182,7 +180,7 @@ pl_metacut(void)
 }
 
 
-#if O_SECURE
+#if O_SECURE || O_DEBUG
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 checkData(p) verifies p points to valid  Prolog  data  and  generates  a
@@ -199,8 +197,9 @@ heap).
 #define onLocal(p) onStack(local, p)
 #define onHeap(p) ((char *)p >= (char *)hBase && (char *)p <= (char *)hTop)
 
+int
 trap_gdb()
-{
+{ return 0;
 }
 
 static void
