@@ -1,4 +1,4 @@
-/*  $Id: pl-proc.c,v 1.14 1994/10/19 15:15:09 jan Exp $
+/*  pl-proc.c,v 1.20 1994/12/02 16:40:26 jan Exp
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -15,7 +15,6 @@ finding source files, etc.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 forwards void	resetReferencesModule(Module);
-forwards bool	attribute(Definition, Word, short);
 forwards bool	autoImport(FunctorDef, Module);
 
 SourceFile sourceFileTable = (SourceFile) NULL;
@@ -42,7 +41,7 @@ lookupProcedure(FunctorDef f, Module m)
 
   def->definition.clauses = (Clause) NULL;
   def->lastClause = (Clause) NULL;
-#if O_PROFILE
+#ifdef O_PROFILE
   def->profile_ticks = 0;
   def->profile_calls = 0;
   def->profile_redos = 0;
@@ -59,8 +58,6 @@ resetProcedure(Procedure proc)
 { register Definition def = proc->definition;
 
   def->flags ^= def->flags & ~SPY_ME;	/* Preserve the spy flag */
-  def->source = (SourceFile) NULL;
-  def->source_count = 0;
   set(def, TRACE_ME);
   if ( proc->functor->arity == 0 )
   { def->indexPattern = 0x0;
@@ -90,6 +87,35 @@ isDefinedProcedure(register Procedure proc)
   fail;
 }
 
+
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+Find a procedure for defining it.  Here   we check whether the procedure
+to be defined is a system predicate.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+Procedure
+lookupProcedureToDefine(FunctorDef def, Module m)
+{ Procedure proc;
+
+  if ( (proc = isCurrentProcedure(def, m)) && false(proc->definition, SYSTEM) )
+    return proc;
+
+  if ( !SYSTEM_MODE &&
+       MODULE_system &&
+       (proc=isCurrentProcedure(def, MODULE_system)) &&
+       true(proc->definition, LOCKED) &&
+       false(proc->definition, DYNAMIC) )
+  { warning("Attempt to redefine a system predicate: %s/%d\n"
+	    "\tUse :- redefine_system_predicate(+Head) if this is intended",
+	    stringAtom(proc->functor->name),
+	    proc->functor->arity);
+    return NULL;
+  }
+ 
+  return lookupProcedure(def, m);
+}
+
+
 /*  Find a procedure from description `descr'. `descr' is one of:
     <term> or <module>:<term>. If the procedure does not exists NULL
     is returned.
@@ -261,7 +287,7 @@ abolishProcedure(Procedure proc, Module module)
     def->module = module;
     def->definition.clauses = (Clause) NULL;
     def->lastClause = (Clause) NULL;
-#if O_PROFILE
+#ifdef O_PROFILE
     def->profile_ticks = 0;
     def->profile_calls = 0;
     def->profile_redos = 0;
@@ -281,28 +307,41 @@ abolishProcedure(Procedure proc, Module module)
     succeed;
   }
 
-  removeClausesProcedure(proc);
+  removeClausesProcedure(proc, 0);
   resetProcedure(proc);
 
   succeed;
 }
 
 void
-removeClausesProcedure(Procedure proc)
+removeClausesProcedure(Procedure proc, int sfindex)
 { Definition def = proc->definition;
   Clause c, next;
 
   startCritical;
-  for(c = def->definition.clauses; c; c = next)
+  c = def->definition.clauses;
+  def->definition.clauses = def->lastClause = (Clause) NULL;
+
+  for(; c; c = next)
   { next = c->next;
-    if (c->references == 0)
-    { freeClause(c);
-    } else
-    { set(c, ERASED);
-      c->next = (Clause) NULL;
+
+    if ( sfindex == 0 || sfindex == c->source_no )
+    { if (c->references == 0)
+      { freeClause(c);
+      } else
+      { set(c, ERASED);
+	c->next = (Clause) NULL;
+      }
+    } else				/* keep this clause (multifile) */
+    { if ( !def->lastClause )
+      { def->definition.clauses = def->lastClause = c;
+      } else
+      { def->lastClause->next = c;
+	def->lastClause = c;
+      }
+      c->next = NULL;
     }
   }
-  def->definition.clauses = def->lastClause = (Clause) NULL;
 
   endCritical;
 }
@@ -397,7 +436,7 @@ resetReferencesModule(Module m)
 
   for_table(s, m->procedures)
   { def = ((Procedure) s->value)->definition;
-#if O_PROFILE
+#ifdef O_PROFILE
     clear(def, PROFILE_TICKED);
 #endif /* O_PROFILE */
     if ( true(def, FOREIGN) )
@@ -495,7 +534,9 @@ trapUndefined(Procedure proc)
   { word goal;
     mark m;
     bool rval;
-  
+    Atom sfn = source_file_name;	/* needs better solution! */
+    int  sln = source_line_no;
+
     Mark(m);
     goal = globalFunctor(FUNCTOR_undefinterc3);
     unifyAtomic(argTermP(goal, 0), proc->definition->module->name);
@@ -505,6 +546,8 @@ trapUndefined(Procedure proc)
     debugstatus.suspendTrace++;
     rval = callGoal(MODULE_system, goal, FALSE);
     debugstatus.suspendTrace--;
+    source_file_name = sfn;
+    source_line_no   = sln;
 
     Undo(m);
 
@@ -551,7 +594,7 @@ pl_retract(Word term, word h)
 
   if ((term = stripModule(term, &m)) == (Word) NULL)
     fail;
-
+ 
   if (splitClause(term, &head, &body) == FALSE)
     return warning("retract/1: illegal specification");
 
@@ -568,7 +611,7 @@ pl_retract(Word term, word h)
 
     if ( true(proc->definition, FOREIGN) )
       return warning("retract/1: cannot retract from a foreign predicate");
-    if ( true(proc->definition, SYSTEM) && false(proc->definition, DYNAMIC) )
+    if ( true(proc->definition, LOCKED) && false(proc->definition, DYNAMIC) )
       return warning("retract/1: Attempt to retract from a system predicate");
 
     clause = proc->definition->definition.clauses;
@@ -642,7 +685,7 @@ pl_abolish(Word atom, Word arity)
   if ((proc = isCurrentProcedure(f, m)) == (Procedure) NULL)
     succeed;
 
-  if ( true(proc->definition, SYSTEM) && !SYSTEM_MODE && m == MODULE_system )
+  if ( true(proc->definition, LOCKED) && !SYSTEM_MODE && m == MODULE_system )
     return warning("abolish/2: attempt to abolish a system predicate");
 
   return abolishProcedure(proc, m);
@@ -709,31 +752,39 @@ pl_list_active_procedures(void)
   succeed;
 }
 
-static bool
-attribute(Definition def, Word value, short att)
-{ if ( isVar(*value) )
-    return unifyAtomic(value, consNum((def->flags & att) ? 1 : 0));
 
-  switch((int) valNum(*value))
-  { case 0:	clear(def, att);
-		succeed;
-    case 1:	set(def, att);
-		if ( (att == DYNAMIC || att == MULTIFILE) && SYSTEM_MODE )
-		{ set(def, SYSTEM);
-		  set(def, HIDE_CHILDS);
-		}
-		succeed;
-    default:	return sysError("$predicate_attribute/3: Illegal value");
-  }
+static unsigned long
+attribute_mask(Atom key)
+{
+#define TRACE_ANY (TRACE_CALL|TRACE_REDO|TRACE_EXIT|TRACE_FAIL)
+
+  if (key == ATOM_dynamic)	 return DYNAMIC;
+  if (key == ATOM_multifile)	 return MULTIFILE;
+  if (key == ATOM_system)	 return SYSTEM;
+  if (key == ATOM_locked)	 return LOCKED;
+  if (key == ATOM_spy)		 return SPY_ME;
+  if (key == ATOM_trace)	 return TRACE_ME;
+  if (key == ATOM_trace_call)	 return TRACE_CALL;
+  if (key == ATOM_trace_redo)	 return TRACE_REDO;
+  if (key == ATOM_trace_exit)	 return TRACE_EXIT;
+  if (key == ATOM_trace_fail)	 return TRACE_FAIL;
+  if (key == ATOM_trace_any)	 return TRACE_ANY;
+  if (key == ATOM_hide_childs)	 return HIDE_CHILDS;
+  if (key == ATOM_transparent)	 return TRANSPARENT;
+  if (key == ATOM_discontiguous) return DISCONTIGUOUS;
+
+  return 0;
 }
 
+
 word
-pl_predicate_attribute(Word pred, Word what, Word value)
+pl_get_predicate_attribute(Word pred, Word what, Word value)
 { Procedure proc;
   FunctorDef fd;
   Definition def;
   Atom key;
   Module module = (Module) NULL;
+  unsigned long att;
 
   pred = stripModule(pred, &module);
   if ( isAtom(*pred) )
@@ -743,57 +794,90 @@ pl_predicate_attribute(Word pred, Word what, Word value)
   else
     fail;
 
-  proc = (isVar(*value) ? resolveProcedure(fd, module)  /* lookup */
-		        : lookupProcedure(fd, module)); /* set */
+  proc = resolveProcedure(fd, module);
   def = proc->definition;
 
   if (!isAtom(*what) )
-    return warning("$predicate_attribute/3: key should be an atom");
+    return warning("$get_predicate_attribute/3: key should be an atom");
   key = (Atom) *what;
 
   if (key == ATOM_imported)
   { if (module == def->module)
       fail;
     return unifyAtomic(value, def->module->name);
-  }
-  if (key == ATOM_indexed)
+  } else if (key == ATOM_indexed)
   { if (def->indexPattern == 0x0)
       fail;
     return indexPatternToTerm(proc, value);
-  }
+  } else if (key == ATOM_exported)
+  { return unifyAtomic(value, consNum(isPublicModule(module, proc)));
+  } else if (key == ATOM_defined)
+  { return unifyAtomic(value, consNum(true(def, FOREIGN) ||
+				      def->definition.clauses ? 1 : 0));
+  } else if (key == ATOM_line_count)
+  { int line;
 
-  if (!isVar(*value) && (!isInteger(*value) || (valNum(*value) & ~1)))
-    return warning("$predicate_attribute/3: illegal 3rd argument");
-
-#define TRACE_ANY (TRACE_CALL|TRACE_REDO|TRACE_EXIT|TRACE_FAIL)
-
-  if (key == ATOM_dynamic)	return attribute(def, value, DYNAMIC);
-  if (key == ATOM_multifile)	return attribute(def, value, MULTIFILE);
-  if (key == ATOM_system)	return attribute(def, value, SYSTEM);
-  if (key == ATOM_spy)		return attribute(def, value, SPY_ME);
-  if (key == ATOM_trace)	return attribute(def, value, TRACE_ME);
-  if (key == ATOM_trace_call)	return attribute(def, value, TRACE_CALL);
-  if (key == ATOM_trace_redo)	return attribute(def, value, TRACE_REDO);
-  if (key == ATOM_trace_exit)	return attribute(def, value, TRACE_EXIT);
-  if (key == ATOM_trace_fail)	return attribute(def, value, TRACE_FAIL);
-  if (key == ATOM_trace_any)	return attribute(def, value, TRACE_ANY);
-  if (key == ATOM_hide_childs)	return attribute(def, value, HIDE_CHILDS);
-  if (key == ATOM_transparent)	return attribute(def, value, TRANSPARENT);
-  if (key == ATOM_discontiguous) return attribute(def,value, DISCONTIGUOUS);
-  if (key == ATOM_foreign)
-    return unifyAtomic(value, consNum(true(def, FOREIGN) ? 1 : 0));
-  if (key == ATOM_exported)
-    return unifyAtomic(value, consNum(isPublicModule(module, proc)));
-  if (key == ATOM_defined)
-    return unifyAtomic(value, consNum(true(def, FOREIGN) ||
-				      def->definition.clauses ? 1 : 0) );
-  if (key == ATOM_line_count)
-  { if ( def->line_no > 0 )
-      return unifyAtomic(value, consNum(def->line_no));
+    if ( false(def, FOREIGN) &&
+	 def->definition.clauses &&
+	 (line=def->definition.clauses->line_no) )
+      return unifyAtomic(value, consNum(line));
     else
       fail;
+  } else if (key == ATOM_foreign)
+  { return unifyAtomic(value, consNum((def->flags & FOREIGN) ? 1 : 0));
+  } else if ( (att = attribute_mask(key)) )
+  { return unifyAtomic(value, consNum((def->flags & att) ? 1 : 0));
   } else
-    return warning("$predicate_attribute/4: unknown key");
+  { return warning("$get_predicate_attribute/4: unknown key: %s",
+		   stringAtom(key));
+  }
+}
+  
+
+word
+pl_set_predicate_attribute(Word pred, Word what, Word value)
+{ Procedure proc;
+  FunctorDef fd;
+  Definition def;
+  Atom key;
+  Module module = (Module) NULL;
+  unsigned long att;
+  int nodef;				/* does not define pred */
+
+  if ( !isAtom(*what) ||
+       (!isInteger(*value) || (valNum(*value) & ~1)) )
+    return warning("$set_predicate_attribute/3: instantiation fault");
+  key = (Atom) *what;
+  if ( !(att = attribute_mask(key)) )
+    return warning("$set_predicate_attribute/4: unknown key: %s",
+		   stringAtom(key));
+
+  nodef = (att & (TRACE_ANY|SPY_ME));
+
+  pred = stripModule(pred, &module);
+  if ( isAtom(*pred) )
+    fd = lookupFunctorDef((Atom) *pred, 0);
+  else if ( isTerm(*pred) )
+    fd = functorTerm(*pred);
+  else
+    fail;
+
+  proc = (nodef ? resolveProcedure(fd, module)
+	        : lookupProcedureToDefine(fd, module));
+  if ( !proc )
+    fail;
+  def = proc->definition;
+
+  if ( *value == consNum(0) )
+  { clear(def, att);
+  } else
+  { set(def, att);
+    if ( (att == DYNAMIC || att == MULTIFILE) && SYSTEM_MODE )
+    { set(def, SYSTEM|HIDE_CHILDS);
+    }
+  }
+
+  succeed;
 }
 
 
@@ -850,6 +934,37 @@ pl_index(Word pred)
   succeed;
 }
 
+
+word
+pl_get_clause_attribute(Word ref, Word att, Word value)
+{ Clause clause;
+  Atom a = (Atom) *att;
+  word result;
+
+  if ( !isInteger(*ref) ||
+       !(clause = (Clause) numToPointer(*ref)) ||
+       !inCore(clause) || !isClause(clause) )
+    return warning("$clause_attribute/3: illegal reference");
+
+  if ( a == ATOM_line_count )
+  { if ( !clause->line_no )
+      fail;
+    else
+      result = (word) consNum(clause->line_no);
+  } else if ( a == ATOM_file )
+  { SourceFile sf = indexToSourceFile(clause->source_no);
+    
+    if ( sf )
+      result = (word) sf->name;
+    else
+      fail;
+  } else
+    fail;
+
+  return unifyAtomic(value, result);
+}
+
+
 		/********************************
 		*         SOURCE FILE           *
 		*********************************/
@@ -857,19 +972,23 @@ pl_index(Word pred)
 SourceFile
 lookupSourceFile(Atom name)
 { SourceFile file;
+  static int index = 0;
 
   for(file=sourceFileTable; file; file=file->next)
   { if (file->name == name)
       return file;
   }
+
   file = (SourceFile) allocHeap(sizeof(struct sourceFile) );
   file->name = name;
   file->count = 0;
   file->time = 0L;
+  file->index = ++index;
   file->system = status.boot;
-  file->next = (SourceFile) NULL;
+  file->procedures = NULL;
+  file->next = NULL;
 
-  if ( sourceFileTable == (SourceFile) NULL )
+  if ( sourceFileTable == NULL )
   { sourceFileTable = tailSourceFileTable = file;
   } else
   { tailSourceFileTable->next = file;
@@ -879,6 +998,7 @@ lookupSourceFile(Atom name)
   return file;
 }
 
+
 SourceFile
 isCurrentSourceFile(Atom name)
 { SourceFile file;
@@ -891,6 +1011,37 @@ isCurrentSourceFile(Atom name)
   return (SourceFile) NULL;
 }
 
+
+SourceFile
+indexToSourceFile(int index)
+{ SourceFile file;
+
+  for(file=sourceFileTable; file; file=file->next)
+  { if (file->index == index)
+      return file;
+  }
+
+  return NULL;
+}
+
+
+void
+addProcedureSourceFile(SourceFile sf, Procedure proc)
+{ ListCell cell;
+
+  for(cell=sf->procedures; cell; cell = cell->next)
+    if ( cell->value == proc )
+      return;
+
+  startCritical;
+  cell = allocHeap(sizeof(struct list_cell));
+  cell->value = proc;
+  cell->next = sf->procedures;
+  sf->procedures = cell;
+  endCritical;
+}
+
+
 word
 pl_make_system_source_files(void)
 { SourceFile file;
@@ -904,13 +1055,17 @@ pl_make_system_source_files(void)
 word
 pl_source_file(Word descr, Word file)
 { Procedure proc;
+  Clause clause;
+  SourceFile sf;
 
-  if ((proc = findProcedure(descr)) == (Procedure) NULL)
-    fail;
-  if (proc->definition->source == (SourceFile) NULL)
+  if ( !(proc = findProcedure(descr)) ||
+       !proc->definition ||
+       true(proc->definition, FOREIGN) ||
+       !(clause = proc->definition->definition.clauses) ||
+       !(sf = indexToSourceFile(clause->source_no)) )
     fail;
 
-  return unifyAtomic(file, proc->definition->source->name);
+  return unifyAtomic(file, sf->name);
 }
 
 word
@@ -951,8 +1106,23 @@ pl_start_consult(Word file)
   if (!isAtom(*file) )
     fail;
   f = lookupSourceFile((Atom)*file);
-  f->count++;
   f->time = LastModifiedFile(stringAtom(*file));
 
+  if ( f->count++ > 0 )
+  { ListCell cell, next;
+
+    for(cell = f->procedures; cell; cell = next)
+    { Procedure proc = cell->value;
+
+      next = cell->next;
+      if ( proc->definition )
+	removeClausesProcedure(proc, true(proc->definition, MULTIFILE)
+						? f->index : 0);
+      freeHeap(cell, sizeof(struct list_cell));
+    }
+    f->procedures = NULL;
+  }
+  f->current_procedure = NULL;
+
   succeed;
 }
