@@ -1,4 +1,4 @@
-/*  $Id: pl-trace.c,v 1.11 1994/06/20 10:44:56 jan Exp $
+/*  pl-trace.c,v 1.15 1994/12/02 16:40:27 jan Exp
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -358,7 +358,7 @@ traceAction(char *cmd, int port, LocalFrame frame, bool interactive)
     case '\04':
     case EOF:	FeedBack("EOF: ");
     case 'e':	FeedBack("exit\n");
-		pl_halt();
+		Halt(0);
     case 'f':	FeedBack("fail\n");
 		return ACTION_FAIL;
     case 'i':	if (port & (CALL_PORT|REDO_PORT|FAIL_PORT))
@@ -725,11 +725,12 @@ again:
   ResetTty();                           /* clear pending input -- atoenne -- */
   c = getSingleChar();
 
-#if O_SIG_AUTO_RESET
-#if OS2 && EMX
+#ifndef BSD_SIGNALS
+#ifdef SIG_ACK
   signal(SIGINT, SIG_ACK);
-#endif
+#else
   signal(SIGINT, interruptHandler);	/* reinsert handler */
+#endif
 #endif
 
   switch(c)
@@ -744,7 +745,7 @@ again:
     case 04:
     case EOF:	Putf("EOF: ");
     case 'e':	Putf("exit\n");
-		pl_halt();
+		Halt(0);
 		break;
     case 'g':	Putf("goals\n");
 		backTrace(environment_frame, 5);
@@ -765,7 +766,7 @@ again:
 void
 initTracer(void)
 { 
-#if O_SIGNAL
+#ifdef SIGINT
   pl_signal(SIGINT, interruptHandler);
 #endif
 
@@ -933,11 +934,17 @@ pl_prolog_frame_attribute(Word frame, Word what, Word value)
 
     if ((parent = parentFrame(fr)) != (LocalFrame) NULL)
       result = PrologRef(parent);
-    fail;
+    else
+      fail;
   } else if (key == ATOM_top)
   { result = consNum(fr->parent == (LocalFrame) NULL ? 1 : 0);
   } else if (key == ATOM_context_module)
   { result = (word) contextModule(fr)->name;
+  } else if (key == ATOM_clause)
+  { if ( false(fr->procedure->definition, FOREIGN) && fr->clause )
+      result = pointerToNum(fr->clause);
+    else
+      fail;
   } else if (key == ATOM_goal)
   { int arity, n;
     Word arg;
