@@ -1,4 +1,4 @@
-/*  $Id: pl-write.c,v 1.7 1994/03/02 14:33:34 jan Exp $
+/*  pl-write.c,v 1.8 1994/11/11 12:52:32 jan Exp
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -118,8 +118,13 @@ writePrimitive(Word w, bool quote)
 	case AT_SYMBOL:
 	case AT_SOLO:
 	case AT_SPECIAL:
-	  Putf("%s", stringAtom(*w));
+	{ char *s = stringAtom(*w);
+
+	  for(; *s; s++)
+	    Put(*s);
+
           break;
+	}
         case AT_QUOTE:
         case AT_FULLSTOP:
 	default:
